<?php

namespace App\Factory;

use App\Exception;

class OrderFactory {
    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function insertSale ($nrvendarest, $generalParams, $consumer) {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    	
    	$params = [
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $nrvendarest,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":NRMESA"       => $generalParams["NRMESA"],
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN" => "1",
            ":CDCLIENTE"    => $generalParams["CDCLIENTE"],
            ":CDCONSUMIDOR" => $generalParams["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function saveComandaven($cdfilial, $nrvendarest, $nrcomanda, $nrcomandaext, $cdloja, $dscomanda, $idstcomanda,
                                    $deliveryFee, $consumerData, $dsobscomanda, $nrpedmobile, $vrdesccomanda, $idRetBalLoja, $deliveryMethod, $nrContatoCon) {
                                        
        $consumerName = !empty($consumerData['consumerInfo']) ? substr(($consumerData['consumerInfo']['first_name'] . " " .$consumerData['consumerInfo']['last_name']), 0, 50) : null;
        $sql = "
            INSERT INTO COMANDAVEN 
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    	        CDLOJA, DSCOMANDA, IDSTCOMANDA, 
    	        VRACRCOMANDA, VRDESCOMANDA, CDPAIS, SGESTADO, 
    	        CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, NRENDECONSCOMAN,
    	        DSENDECONSCOMAN, DSCOMPLENDCOCOM, DSREFENDCONSCOM, DSOBSCOMANDA,
    	        IDORGCMDVENDA, DSCONSUMIDOR, NRPEDMOBILE, IDRETBALLOJA, IDENTRPLAT, NRCONTATOCON)
    	    VALUES 
    	        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    	        :CDLOJA, :DSCOMANDA, :IDSTCOMANDA, 
    	        :VRACRCOMANDA, :VRDESCOMANDA, :CDPAIS, :SGESTADO, 
    	        :CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, :NRENDECONSCOMAN,
    	        :DSENDECONSCOMAN, :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :DSOBSCOMANDA,
    	        :IDORGCMDVENDA, :DSCONSUMIDOR, :NRPEDMOBILE, :IDRETBALLOJA, :IDENTRPLAT, :NRCONTATOCON)";
    	
    	
    	$params = [
    	    ":CDFILIAL"        => $cdfilial,
            ":NRVENDAREST"     => $nrvendarest,
            ":NRCOMANDA"       => $nrcomanda,
            ":NRCOMANDAEXT"    => $nrcomandaext,
            ":CDLOJA"          => $cdloja,
            ":DSCOMANDA"       => $dscomanda,
            ":IDSTCOMANDA"     => $idstcomanda,
            ":VRACRCOMANDA"    => $this->util->toFloat($deliveryFee),
            ":VRDESCOMANDA"    => $this->util->toFloat($vrdesccomanda),
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => $consumerData['SGESTADO'],
            ":CDMUNICIPIO"     => $consumerData['CDMUNICIPIO'],
            ":NRCEPCONSCOMAND" => $consumerData['NRCEPCONS'],
            ":CDBAIRRO"        => $consumerData['CDBAIRRO'],
            ":DSBAIRRO"        => $consumerData['NMBAIRCONS'],
            ":NRENDECONSCOMAN" => $consumerData['NRENDECONS'],
            ":DSENDECONSCOMAN" => substr($consumerData['DSENDECONS'], 0, 50),
            ":DSCOMPLENDCOCOM" => $consumerData['DSCOMPLENDECONS'],
            ":DSREFENDCONSCOM" => $consumerData['DSREFERENDECONS'],
            ":DSOBSCOMANDA"    => $dsobscomanda,
            ":IDORGCMDVENDA"   => "DLV_RAP",
            ":DSCONSUMIDOR"    => $consumerName,
            ":NRPEDMOBILE"     => 'RAP'.$nrpedmobile,
            ":IDRETBALLOJA"    => $idRetBalLoja,
            ":IDENTRPLAT"      => $deliveryMethod,
            ":NRCONTATOCON"    => $nrContatoCon
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment ($order, $value, $generalParams, $consumer) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $order["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $order["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $generalParams["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $generalParams["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $order["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->util->toFloat($value),
	        ":NRCARTBANCODLV"  => null
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function validateDuplication ($generalParams, $orderID) {
        $sql = "
            SELECT C.NRPEDMOBILE
    		  FROM COMANDAVEN C
    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST = C.NRVENDAREST
    		   AND V.CDFILIAL = C.CDFILIAL
    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND C.IDORGCMDVENDA = 'DLV_RAP'
    		   AND V.DTHRABERMESA >= SYSDATE - 1 
        ";
        
        $params = [
            ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRPEDMOBILE" => 'RAP'.$orderID
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        return !empty($result);
    }
    
    public function disableIntegration() {
        $sql = "
            UPDATE INTEGRACAO
               SET IDHABINTE    = 'N'
             WHERE IDINTEGRACAO = 'RAP'
        ";
        
        $this->connection->executeQuery($sql);
    }

    public function getConfiguration() {
        $sql = "
            SELECT IDUFTIMEZONE, IDTOKENINTGERAL, IDSENHATOKENGERAL, IDHABINTE, AUTHTOKEN, AUTHTKNDTCRIAC
              FROM INTEGRACAO
             WHERE IDINTEGRACAO = 'RAP'
        ";
        
        return $this->connection->fetchAssoc($sql);
    }
    
    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest) {
        $taxSQL = "
            SELECT VRACRCOMANDA, VRDESCOMANDA 
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);
        
        $discountSum = isset($taxData['VRDESCOMANDA']) ? (string)$this->formatNumber($taxData['VRDESCOMANDA']) : "0.00";
        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 2);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 2);
            $itemsSum = bcadd($itemsSum, $sub, 2);
        }

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $subOrderSum   = bcadd($itemsSum, $taxSum, 2);
        $totalOrderSum = bcsub($subOrderSum, $discountSum, 2);
        $paymentsSum   = bcsub($entradasSum, $saidasSum, 2);
        
        if(bccomp($totalOrderSum, "0", 2) <= 0 || bccomp($paymentsSum, "0", 2) <= 0) {
            throw new Exception\ParamsException("Não foi possível processar o pedido, desconto de 100% ou valor pago ZERADO.<br>Descontos ($discountSum) - Total de Itens ($subOrderSum) = $totalOrderSum | Valor pago = $paymentsSum");
        }
        
        if(bccomp($totalOrderSum, $paymentsSum) != 0) {
            throw new Exception\ParamsException("Não foi possível processar o pedido, existe uma divergencia entre os itens do pedido e o valor pago.<br>Valor dos itens: $totalOrderSum | Valor pago: $paymentsSum");
        }
    }
    
    private function formatNumber($number) {
        return str_replace(",", ".", $number);
    }
    
    public function updateAccessToken($token) {
        $sql = "
            UPDATE INTEGRACAO
               SET AUTHTOKEN      = :AUTHTOKEN,
                   AUTHTKNDTCRIAC = :AUTHTKNDTCRIAC
             WHERE IDINTEGRACAO   = 'RAP'
        ";
    	
    	$params = [
    	    ":AUTHTOKEN"      => $token,
            ":AUTHTKNDTCRIAC" => time()
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function getIdUtilSku(){
        $sql = "
            SELECT IDUTILSKU FROM INTEGRACAO
                WHERE IDINTEGRACAO = :IDINTEGRACAO
        ";
        
        $params = [
             ":IDINTEGRACAO" => 'RAP'    
        ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        if($result['IDUTILSKU'] === "S")
            return true;
        
        return false;
    }
}
