<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {

        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function processProduct ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ, $isSku) {
        
        if($product["type"] === 'product' && $product['price'] >= 0.01 ) {
            return $this->processSimpleProduct($generalParams, $product, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ, $isSku);
        
        } else if($product["type"] === 'product' && $product['price'] == 0 ) {
            if(empty($product['subitems']) || count($product['subitems']) == 0) {
                 throw new \Exception("O produto {$product['name']} foi enviado com preço zero e não possui subitens para ser processado como combo.");
            }
            
            return $this->processComboProduct($generalParams, $product, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ, $isSku);
            
        } else {
            throw new \Exception("O produto {$product['name']} (modelo {$product['type']}) não é suportado pela integração, verifique os modelos de produtos suportados no manual da integração.");
        }
        
        //switch($product["type"]) {
        //    case "product":
        //        
        //        break;
        //    
        //    default: 
        //        throw new \Exception("O produto {$product['name']} (modelo {$product['type']}) não é suportado pela integração, verifique os modelos de produtos suportados no manual da integração.");
        //        break;
        //}
        //
        //return $processedProduct;
    }
    
    private function processSimpleProduct ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ, $isSku)
    {
        $productItems = [];
        if($isSku) {
            $productDetails = $this->factory->getProduct($product["sku"]);
            if (empty($productDetails)) 
                throw new Exception\ParamsException("
                    O produto Cod Teknisa: {$product['sku']} - {$product['name']} não encontrado. Verifique o código cadastrado no portal rappi.<br>
                    Para fazer a correção, acesse o Odhen Retail, abra tela Parâmetro de Integração e Aplicativos.<br>
                    Escolha a integração Rappi, clique na aba Mapeamento de Produto e faça a correção.<br>
                    Para mais informações, revise o Capítulo 3, no manual da integração.<br>
                    Em caso de dúvidas, entre em contato com a equipe de suporte da Teknisa.<br>");
        } else{
            $productDetails = $this->factory->getProductMap($product["id"]);
            if (empty($productDetails)) 
                throw new Exception\ParamsException(
                    "O produto  Cod Rappi: {$product['id']} - {$product['name']} não está mapeado com o código de produto Teknisa.<br>
                    Para fazer a correção, acesse o Odhen Retail, abra tela Parâmetro de Integração e Aplicativos.<br>
                    Escolha a integração Rappi, clique na aba Mapeamento de Produto e faça a correção.<br>
                    Para mais informações, revise o Capítulo 3, no manual da integração.<br>
                    Em caso de dúvidas, entre em contato com a equipe de suporte da Teknisa.<br>");
        }

        $productDetails = array_merge($productDetails, $product);
        
        for($i = 0; $i < $product["quantity"]; $i++) {
            $nrSeqProdComPai = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            $productDetails["comments"] = (isset($productDetails["comments"]) && !empty($productDetails["comments"])) ? (" {$productDetails['comments']}; ") : "";
            
            $pruductSimpleTemplate = [
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $nrvendarest, 
                "NRCOMANDA"        => $nrcomanda,
                "NRPRODCOMVEN"     => $nrSeqProdComPai,
                "CDPRODUTO"        => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => $productDetails["price"], 
                "TXPRODCOMVEN"     => $productDetails["comments"],
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                "VRDESCCOMVEN"     => !empty($productDetails["discount"]) ? $productDetails["discount"] : 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => null,
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"   => null,
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => []
            ];

            foreach ($product["subitems"] as $item) {
                if($isSku) {
                    if($item['price'] >= 0.01) {
                        if (strlen($item['sku']) === 5 && substr($item['sku'], 2, 1) === '|') {
                            $observationDetails = $this->factory->getOcorencia($item["sku"], true);
                            if(isset($itemDetails['IDCONTROLAOBS']) && $itemDetails['IDCONTROLAOBS'] != "A" && strpos($comboItem['customCode'], "|")) {
                                throw new Exception\ParamsException("
                                    O subitem sku: {$item['sku']} - {$item['name']} foi cadastrado como um acréscimo utilizando o código no padrão XX|YY, mas o produto encontrado ({$observationDetails['DSOCORR']}) foi uma observação.<br>
                                    Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item e/ou tipo de observação no sistema Retail.<br>
                                    Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                    O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                    Para mais informações, consulte a seção <b>Parametrização do Cardápio</b>, no manual da Rappi.");
                            }
                        } else{
                            $observationDetails = $this->factory->getProduct($item["sku"]);
                            if (empty($observationDetails)) {
                                throw new Exception\ParamsException("
                                    O subitem {$item['sku']} - {$item['name']} foi cadastrado como um acréscimo utilizando o código de produto da arvore de produtos(Teknisa), porém não foi encontrado nenhum produto com o código informado.<br>
                                    Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item para o formato XX|YY, e/ou tipo de observação no sistema Retail.<br>
                                    Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                    O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                    Para mais informações, consulte a seção <b>Parametrização do Cardápio</b>, no manual da Rappi.");
                            }
                        }
                            
                        for($y = 0; $y < $item['quantity']; $y++){
                            $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                            $ITCOMANDAVEN_SEQ++;
                            
                            $obsReturn = array(
                                "CDFILIAL"       => $generalParams["CDFILIAL"],
                                "NRVENDAREST"    => $nrvendarest,
                                "NRCOMANDA"      => $nrcomanda,
                                "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"   => 1,
                                "VRPRECCOMVEN"   => $item['price'], 
                                "TXPRODCOMVEN"   => " ",
                                "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                                "VRDESCCOMVEN"   => 0,
                                "NRLUGARMESA"    => "01",
                                "IDPRODIMPFIS"   => "N",
                                "CDLOJA"         => $generalParams["CDLOJA"],
                                "VRACRCOMVEN"    => 0,
                                "NRSEQPRODCOM"   => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                                "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                                "VRPRECCLCOMVEN" => 0,
                                "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                                "CDPRODPROMOCAO" => null,
                                "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                                "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                                "DSOBSPEDDIGCMD" => "",
                                
                            );
                            
                            $pruductSimpleTemplate["NRSEQPRODCOM"] = $this->util->formatNrSeqProdCom($nrSeqProdComPai);
                            array_push($productItems, $obsReturn); 
                        }
                        
                        if (strlen($item['sku']) === 5 && substr($item['sku'], 2, 1) === '|'){
                            array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $generalParams["CDFILIAL"],  
                                'NRVENDAREST'     => $nrvendarest, 
                                'NRCOMANDA'       => $nrcomanda, 
                                'NRPRODCOMVEN'    => $nrSeqProdComPai,  
                                'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                                'CDOCORR'         => $observationDetails['CDOCORR'],  
                                'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                                'QTPRODCOMVENOBS' => $item['quantity']
                            ]);   
                        }
                    } else {
                        if(!isset($item['sku']) || empty($item['sku'])) {
                            throw new Exception\ParamsException("
                                O subitem {$item['name']}, foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'x.xx.xx.xxx.xx' ou no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Rappi.");
                        }
                        
                        if(!strpos($item['sku'], "|")) {
                            throw new Exception\ParamsException("
                                O subitem {$item['name']}, foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$item['sku']}.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Rappi.");
                        }
                        
                        $observationDetails = $this->factory->getOcorencia($item["sku"]);
                        
                        if($observationDetails['IDCONTROLAOBS'] === "A") {
                            throw new Exception\ParamsException("
                                O subitem {$item['name']}, foi cadastrado no sistema Retail como um acréscimo utilizando o código no padrão XX|YY, mas está sem preço.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Rappi.");

                        }
                        if (empty($observationDetails)) {
                            throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$item['sku']}");
                        }
                        
                        $pruductSimpleTemplate['TXPRODCOMVEN'] .= strtoupper("{$observationDetails['DSOCORR']}; ");
                        
                        array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $nrvendarest, 
                            'NRCOMANDA'       => $nrcomanda, 
                            'NRPRODCOMVEN'    => $nrSeqProdComPai,  
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                            'CDOCORR'         => $observationDetails['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => 1
                        ]);
                    }
                } else {
                    if($item['price'] >= 0.01) {
                        $validateProd = $this->factory->getProductMap($item["id"]);
                        
                        if (empty($validateProd)) 
                            throw new Exception\ParamsException("
                                O produto Cod Rappi: {$item['id']} - {$item['name']} não está mapeado com o código de produto Teknisa.<br>
                                Para fazer a correção, acesse o Odhen Retail, abra tela Parâmetro de Integração e Aplicativos.
                                Escolha a integração Rappi, clique na aba Mapeamento de Produto e faça a correção.
                                Para mais informações, revise o Capítulo 3, no manual da integração.
                                Em caso de dúvidas, entre em contato com a equipe de suporte da Teknisa.");
                        
                        for($y = 0; $y < $item["quantity"]; $y++) {
                            $itemDetails = array_merge($validateProd, $item);
        
                            $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                            $ITCOMANDAVEN_SEQ++;
                            
                            $itemReturn = [
                                "CDFILIAL"          => $generalParams["CDFILIAL"],
                                "NRVENDAREST"       => $nrvendarest, 
                                "NRCOMANDA"         => $nrcomanda,
                                "NRPRODCOMVEN"      => $itemDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"         => $itemDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"      => 1,
                                "VRPRECCOMVEN"      => $itemDetails["price"],
                                "TXPRODCOMVEN"      => "",
                                "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                                "VRDESCCOMVEN"      => !empty($item["discount"]) ? $item["discount"] : 0,
                                "NRLUGARMESA"       => "01",
                                "IDPRODIMPFIS"      => "N",
                                "CDLOJA"            => $generalParams["CDLOJA"],
                                "VRACRCOMVEN"       => 0,
                                "NRSEQPRODCOM"      => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                                "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                                "VRPRECCLCOMVEN"    => 0,
                                "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                                "CDPRODPROMOCAO"    => null,
                                "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                                "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                                "DSOBSPEDDIGCMD"    => null,
                                "ITEM_OBSERVATION"  => [] //
                            ];
                            
                            $pruductSimpleTemplate["NRSEQPRODCOM"] = $this->util->formatNrSeqProdCom($nrSeqProdComPai);
                            array_push($productItems, $itemReturn);
                        }
    
                    } else {
    
                        $mapProd      = $this->factory->getProductMap($item["id"]);
                        $validateProd = $this->factory->getObservation($mapProd['CDPRODUTO'], $generalParams["CDFILIAL"], $generalParams["CDLOJA"]);
    
                        if(empty($pruductSimpleTemplate['TXPRODCOMVEN'])){
                            $pruductSimpleTemplate['TXPRODCOMVEN'] = ' ';
                        }
                        
                        if(empty($validateProd)) {
                            $pruductSimpleTemplate['TXPRODCOMVEN'] .= strtoupper("{$item['name']}; ");
                        } else {
                            $pruductSimpleTemplate['TXPRODCOMVEN'] .= strtoupper("{$validateProd['DSOCORR']}; ");
                            array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $generalParams["CDFILIAL"],
                                'NRVENDAREST'     => $nrvendarest,
                                'NRCOMANDA'       => $nrcomanda,
                                'NRPRODCOMVEN'    => $nrSeqProdComPai,
                                'CDGRPOCOR'       => $validateProd['CDGRPOCOR'],
                                'CDOCORR'         => $validateProd['CDOCORR'],
                                'NRPRODCOMVENOBS' => null,
                                'QTPRODCOMVENOBS' => $item['quantity']
                            ]);
                        }
                    }
                }
            }

            array_push($productItems, $pruductSimpleTemplate);
            
        }
        return $productItems;
    }
    
    private function processComboProduct ($generalParams, $product, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ, $isSku)
    {
        $productItems = [];
        
        foreach($product['subitems'] as $item) {
            if($item['price'] == 0) {
                throw new Exception\ParamsException("
                    O produto Cod Rappi: {$product['id']} - {$product['name']} esta sendo processado como combo onde o produto principal esta com preço zero.<br>
                    Nesta configuração de combo é considerado apenas os subitens no pedido. Impossibilitando processamento de observações ou produtos com preço zero como subitem.<br>
                    Para mais informações, revise o manual da integração.<br>
                    Em caso de dúvidas, entre em contato com a equipe de suporte da Teknisa.<br>");
            }
            
            if(!empty($item['sku']) && (strlen($item['sku']) === 5 && substr($item['sku'], 2, 1) === '|')) {
                throw new Exception\ParamsException("
                    O produto Cod Rappi: {$product['id']} - {$product['name']} esta sendo processado como combo onde o produto principal esta com preço zero.<br>
                    Nesta configuração de combo é considerado apenas os subitens no pedido. Impossibilitando processamento de SKU no formato XX|YY como subitem.<br>
                    Para esta configuração de combo é obrigatório que os SKU's dos subitens tenham o formato de código de produto ('X.XX.XXX.XX') e com preço igual ou superior a R$ 0,01.<br>
                    Para mais informações, revise o manual da integração.<br>
                    Em caso de dúvidas, entre em contato com a equipe de suporte da Teknisa.<br>");
            }
            
            $itensCombo = $this->processSimpleProduct($generalParams, $item, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ, $isSku);
            array_push($productItems, $itensCombo[0]);
        }
        
        return $productItems;
    }

    public function insertProduct ($product) {
        $this->factory->insertProduct($product);
        
        if(!empty($product["ITEM_OBSERVATION"])) {
            foreach($product["ITEM_OBSERVATION"] as $obsItem) {
                $this->factory->insertObservation($obsItem);
            }
        }
    }
    
}
